@@ -16,6 +16,7 @@
 #include "shmmc.h"
 #include "pipe.h"
 #include "orafunc.h"
+#include "builtins.h"
 
 /*
  * @ Pavel Stehule 2006
@@ -39,32 +40,6 @@
 #define NOT_INITIALIZED -1
 #define ONE_YEAR (60*60*24*365)
 
-Datum dbms_pipe_pack_message_text(PG_FUNCTION_ARGS);
-Datum dbms_pipe_unpack_message_text(PG_FUNCTION_ARGS);
-Datum dbms_pipe_send_message(PG_FUNCTION_ARGS);
-Datum dbms_pipe_receive_message(PG_FUNCTION_ARGS);
-Datum dbms_pipe_unique_session_name (PG_FUNCTION_ARGS);
-Datum dbms_pipe_list_pipes (PG_FUNCTION_ARGS);
-Datum dbms_pipe_next_item_type (PG_FUNCTION_ARGS);
-Datum dbms_pipe_create_pipe(PG_FUNCTION_ARGS);
-Datum dbms_pipe_create_pipe_2(PG_FUNCTION_ARGS);
-Datum dbms_pipe_create_pipe_1(PG_FUNCTION_ARGS);
-Datum dbms_pipe_reset_buffer(PG_FUNCTION_ARGS);
-Datum dbms_pipe_purge(PG_FUNCTION_ARGS);
-Datum dbms_pipe_remove_pipe(PG_FUNCTION_ARGS);
-Datum dbms_pipe_pack_message_date(PG_FUNCTION_ARGS);
-Datum dbms_pipe_unpack_message_date(PG_FUNCTION_ARGS);
-Datum dbms_pipe_pack_message_timestamp(PG_FUNCTION_ARGS);
-Datum dbms_pipe_unpack_message_timestamp(PG_FUNCTION_ARGS);
-Datum dbms_pipe_pack_message_number(PG_FUNCTION_ARGS);
-Datum dbms_pipe_unpack_message_number(PG_FUNCTION_ARGS);
-Datum dbms_pipe_pack_message_bytea(PG_FUNCTION_ARGS);
-Datum dbms_pipe_unpack_message_bytea(PG_FUNCTION_ARGS);
-Datum dbms_pipe_pack_message_record(PG_FUNCTION_ARGS);
-Datum dbms_pipe_unpack_message_record(PG_FUNCTION_ARGS);
-Datum dbms_pipe_pack_message_integer(PG_FUNCTION_ARGS);
-Datum dbms_pipe_pack_message_bigint(PG_FUNCTION_ARGS);
-
 PG_FUNCTION_INFO_V1(dbms_pipe_pack_message_text);
 PG_FUNCTION_INFO_V1(dbms_pipe_unpack_message_text);
 PG_FUNCTION_INFO_V1(dbms_pipe_send_message);
@@ -183,7 +158,7 @@ pack_field(message_buffer *buffer, message_data_type type,
 		ereport(ERROR,
 				(errcode(ERRCODE_OUT_OF_MEMORY),
 				 errmsg("out of memory"),
-				 errdetail("Packed message is biger than local buffer."),
+				 errdetail("Packed message is bigger than local buffer."),
 				 errhint("Increase LOCALMSGSZ in 'pipe.h' and recompile library.")));
 
 	if (buffer->next == NULL)
@@ -662,7 +637,11 @@ dbms_pipe_pack_message_record(PG_FUNCTION_ARGS)
 	 * using fcinfo->flinfo->fn_extra.  So we need to pass it our own
 	 * flinfo parameter.
 	 */
+#if PG_VERSION_NUM >= 90100
+	InitFunctionCallInfoData(info, fcinfo->flinfo, 3, InvalidOid, NULL, NULL);
+#else
 	InitFunctionCallInfoData(info, fcinfo->flinfo, 3, NULL, NULL);
+#endif
 
 	info.arg[0] = PointerGetDatum(rec);
 	info.arg[1] = ObjectIdGetDatum(tupType);
@@ -737,7 +716,11 @@ dbms_pipe_unpack_message(PG_FUNCTION_ARGS, message_data_type dtype)
 			 * using fcinfo->flinfo->fn_extra.  So we need to pass it our own
 			 * flinfo parameter.
 			 */
+#if PG_VERSION_NUM >= 90100
+			InitFunctionCallInfoData(info, fcinfo->flinfo, 3, InvalidOid, NULL, NULL);
+#else
 			InitFunctionCallInfoData(info, fcinfo->flinfo, 3, NULL, NULL);
+#endif
 
 			info.arg[0] = PointerGetDatum(&buf);
 			info.arg[1] = ObjectIdGetDatum(tupType);
