@@ -19,13 +19,12 @@
 #include "stdlib.h"
 #include "utils/pg_locale.h"
 #include "mb/pg_wchar.h"
-
+#include "nodes/execnodes.h"
 
 #include "catalog/pg_type.h"
 #include "libpq/pqformat.h"
 #include "orafunc.h"
-
-#include "plvstr.h"
+#include "builtins.h"
 
 PG_FUNCTION_INFO_V1(plvstr_rvrs);
 PG_FUNCTION_INFO_V1(plvstr_normalize);
@@ -95,6 +94,7 @@ typedef enum
 	LAST
 }  position_mode;
 
+
 #if PG_VERSION_NUM < 80400
 text *
 cstring_to_text_with_len(const char *c, int n)
@@ -594,7 +594,7 @@ plvstr_rvrs(PG_FUNCTION_ARGS)
 	end = PG_ARGISNULL(2) ? (start < 0 ? -len : len) : end;
 
 	if ((start > end && start > 0) || (start < end && start < 0))
-		PARAMETER_ERROR("Second parameter is biger than third.");
+		PARAMETER_ERROR("Second parameter is bigger than third.");
 
 	if (start < 0)
 	{
