@@ -171,7 +171,6 @@ select concat(1, 'b') = '1b';
 select concat(1, 2) = '12';
 select concat(1, NULL) = '1';
 select concat(NULL, 2) = '2';
-select concat(NULL, NULL) IS NULL;
 select nvl('A'::text, 'B');
 select nvl(NULL::text, 'B');
 select nvl(NULL::text, NULL);
@@ -269,3 +268,8 @@ SELECT dump(10.23::float8) ~ E'^Typ=701 Len=8: \\d+(,\\d+){7}$' AS t;
 SELECT dump(10.23::numeric) ~ E'^Typ=1700 Len=(\\d+): \\d+(,\\d+)*$' AS t;
 SELECT dump('2008-10-10'::date) ~ E'^Typ=1082 Len=4: \\d+(,\\d+){3}$' AS t;
 SELECT dump('2008-10-10'::timestamp) ~ E'^Typ=1114 Len=8: \\d+(,\\d+){7}$' AS t;
+
+select listagg(i::text) from generate_series(1,3) g(i);
+select listagg(i::text, ',') from generate_series(1,3) g(i);
+select coalesce(listagg(i::text), '<NULL>') from (SELECT ''::text) g(i);
+select coalesce(listagg(i::text), '<NULL>') from generate_series(1,0) g(i);
